

#ifdef GL_ES
	precision highp float;
	precision mediump int;
#endif

/// <summary>
/// Uniform variables.
/// <summary>
uniform vec2 ImageSize;
uniform vec2 TexelSize;
uniform float Time;
uniform sampler2D Sample0;

/// <summary>
/// Varying variables.
/// <summary>
varying vec2 vUv;


float height(vec2 p)
{
	return (4.0*sin(p.x/4.0)*sin(p.y/6.0))+(0.25 * cos(p.x)*cos(p.y));
}
const float step = 0.1;

vec3 getNormal(vec2 p )
{
    vec3 n = vec3( height(vec2(p.x-step,p.y)) - height(vec2(p.x+step,p.y)),
                         step,
                         height(vec2(p.x,p.y-step)) - height(vec2(p.x,p.y+step)) );
    return normalize( n );
}
vec3 applyFog(vec3  rgb, float distance )
{
    float fogAmount = min(1.0,1.0 - exp( -distance*0.01 ));
    vec3  fogColor  = vec3(0.5,0.6,0.7);
    return mix( rgb, fogColor, fogAmount );
}
const float cFov = 60.0 * 3.141592 / 180.0;

void main(void)
{
	vec4 result = vec4(0.0,0.0,0.0,1.0);
	vec3 start = vec3(0.0,10.0 + 5.0*sin(Time*0.1),0.0);
	vec3 ray = normalize(vec3(vUv.xy * 2.0 - 1.0, 1.0 / tan(cFov * 0.5)));
	float h;
	int done = 0;
	float lh = 0.0;
    float ly = 0.0;
	for (float t=10.0; t<100.0; t+=5.0)
	{
		if (done==0)
		{
			vec3 p = start + (ray * t);
			h = height(p.xz);
			if ((h>p.y)&&(done==0))
			{
				float resT = t - 5.0 + 5.0*(lh-ly)/(p.y-ly-h+lh);
				p = vec3(0.0,10.0,0.0) + (ray * resT);
				h = height(p.xz);
				vec3 n = getNormal(p.xz);
				vec3 colour = mix(vec3(0.5,0.35,0.0),vec3(0.9,0.9,0.3),h*0.25);
				float light = dot(n,normalize(vec3(0.5,1.0,0.0)));
				result.xyz=colour*light;
				result.xyz=applyFog(result.xyz,p.z);
				done=1;
			}
			lh = h;
			ly = p.y;
		}
	}
	if (done == 0)
	{
		result.xyz = mix(vec3(0.8,0.8,1.0),vec3(0.0,0.0,0.95),vUv.y);
		vec2 sp = vec2(vUv.x-0.95,vUv.y-0.95);
		float s = length(sp);
		s = 0.1/s;
		result.xyz += vec3(1.0,1.0,0.0)*s;
	}
	gl_FragColor = result;

}
